"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const platform_1 = require("@rxstack/platform");
const _ = require("lodash");
const utils_1 = require("./utils");
const exceptions_1 = require("@rxstack/exceptions");
exports.objectExists = (schema) => {
    return (event) => __awaiter(this, void 0, void 0, function* () {
        utils_1.restrictToOperations(event.eventType, [platform_1.OperationEventsEnum.PRE_EXECUTE]);
        yield processData(objectExistsItem, event.request.body, schema, event.injector);
    });
};
const objectExistsItem = (source, schema, injector) => __awaiter(this, void 0, void 0, function* () {
    const data = schema.dataPath ? utils_1.getProperty(source, schema.dataPath) : source;
    yield processData(validateObject, data, schema, injector);
});
const validateObject = (data, schema, injector) => __awaiter(this, void 0, void 0, function* () {
    const id = utils_1.getProperty(data, schema.targetField);
    const service = injector.get(schema.service);
    const defaultCriteria = { [schema.inverseField]: { '$eq': id } };
    const method = schema.method || 'findOne';
    const criteria = schema.criteria ? _.merge(defaultCriteria, schema.criteria) : defaultCriteria;
    const result = yield service[method](criteria);
    if (!result) {
        throw new exceptions_1.BadRequestException('Resource does not exist');
    }
});
const processData = (func, input, schema, injector) => __awaiter(this, void 0, void 0, function* () {
    if (_.isArray(input)) {
        for (let i = 0; i < input.length; i++) {
            yield func(input[i], schema, injector);
        }
    }
    else {
        yield func(input, schema, injector);
    }
});
//# sourceMappingURL=object-exists.js.map