"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const platform_1 = require("@rxstack/platform");
const class_validator_1 = require("class-validator");
const _ = require("lodash");
const exceptions_1 = require("@rxstack/exceptions");
const class_transformer_1 = require("class-transformer");
const utils_1 = require("./utils");
exports.validate = (type, options) => {
    return (event) => __awaiter(this, void 0, void 0, function* () {
        utils_1.restrictToOperations(event.eventType, [platform_1.OperationEventsEnum.PRE_EXECUTE]);
        event.request.body = yield processData(event.request.body, type, resolveOptions(options));
    });
};
const processData = (data, type, options) => __awaiter(this, void 0, void 0, function* () {
    if (_.isArray(data)) {
        for (let i = 0; i < data.length; i++) {
            data[i] = yield validateItem(data[i], type, options);
        }
    }
    else {
        data = yield validateItem(data, type, options);
    }
    return data;
});
const validateItem = (data, type, options) => __awaiter(this, void 0, void 0, function* () {
    const input = _.isString(type) ? data : class_transformer_1.plainToClass(type, data);
    yield doValidate(type, input, options);
    return class_transformer_1.classToPlain(input);
});
const resolveOptions = (options) => {
    const defaults = {
        validationError: { target: false },
        whitelist: true,
        skipMissingProperties: false
    };
    return options ? _.merge(defaults, options) : defaults;
};
const doValidate = (type, input, options) => __awaiter(this, void 0, void 0, function* () {
    const errors = _.isString(type) ? yield class_validator_1.validate(type, input, options)
        : yield class_validator_1.validate(input, options);
    if (errors.length > 0) {
        const exception = new exceptions_1.BadRequestException('Validation Failed');
        exception.data = {
            errors: errors
        };
        throw exception;
    }
});
//# sourceMappingURL=validate.js.map